#! /usr/bin/perl

#a program to use an annotated chromosome file and defined intervals
#analyses:
#total GC content
#intergenic and intronic GC content
#matches and mismatches between D. persimilis MSH3, D. persimilis MSH 1993, and D. miranda

#Define command line inputs: 1) Annotation file name, 2) Interval definitions (tab delimited two column file with start and end positions), and 3) output file name

$annotdata = $ARGV[0];
$intervaldata = $ARGV[1];
$output = $ARGV[2];

#Open all inputs and outputs

open (ANNOTATION, $annotdata);
open (INTERVALS, $intervaldata);
open (OUTPUT, ">$output");

#Read in the interval file (Command line argument 2).
#File should be tab delimited with start and stop positions of each recombination interval
#These intervals should be non-overlapping and the start position should ALWAYS be before the end position!

@start_pos=();
@stop_pos=();

while (<INTERVALS>) {
chomp($interval=$_);
@intervals = split("\t", $interval);

push(@start_pos, $intervals[0]);
push(@stop_pos, $intervals[1]);

} #end while

#Read Annotation File
#Header format should read as follows:
#Chromosome, base position, Dpse_ref, Dper_ref, Bogotana_data, quality, Bogotana_454, Flagstaff_data, quality, Flagstaff_454, NumGenes, Genes, Coding, Type, IntrSize

$first_line=<ANNOTATION>;

@GC_total_count = ();
@AT_total_count = ();
@Dper_mismatch = ();
@Dper_match = ();
@Dmir_mismatch = ();
@Dmir_match = ();
@GC_Intergene_count=();
@AT_Intergene_count=();
@GC_Intron_lt100count=();
@AT_Intron_lt100count=();
@GC_Intron_gt100count=();
@AT_Intron_gt100count=();

$first_interval=1;

for ($j=0; $j<=$#start_pos; $j++) {

while (<ANNOTATION>) {

	$line = $_;
	@line_item=split(",",$line);

#Calculating GC/AT counts

if ($start_pos[$j] <=  $line_item[1] && $stop_pos[$j] >= $line_item[1]) {
        $first_interval=0;
	$base=$line_item[3];

	if ($base eq "A" || $base eq "T") {
		$AT_count++;
	} elsif ($base eq "G" || $base eq "C") {
		$GC_count++;
	} #end elsif

#Calculating matches and mismatches within species for diversity measure

if ($line_item[3] =~ /atgc/ && $line_item[12]=~ /atgc/ && $line_item[11] eq "filtered") {
	if ($line_item[3] eq $line_item[12]) {
		$per_match++;
	} elsif ($line_item[3] ne $line_item[12]) {
		$per_mismatch++;
	} #end if
} #end if

#Calculating matches and mismatches between species for divergence measure

if ($line_item[3] =~ /atgc/ && $line_item[15]=~ /atgc/&& $line_item[14] eq "filtered") {
	if ($line_item[3] eq $line_item[12]) {
		$mir_match++;
	} elsif ($line_item[3] ne $line_item[12]) {
		$mir_mismatch++;
	} #end if
} #end if

#Calculating Intergene GC/AT counts

	if ($line_item[19] eq "Intergene") {
		if ($line_item[3] eq "A" || $line_item[3] eq "T") {				    
		    $interval_ATintergene++;				
		} elsif ($line_item[3] eq "G" || $line_item[3] eq "C") {
		    $interval_GCintergene++;
		} #end if	    

#Calculating Intron GC/AT counts

	} elsif ($line_item[19] eq "Intron") {
		
		if ($line_item[20] > 100) {		
			if ($line_item[3] eq "A" || $line_item[3] eq "T") {				    
			    $interval_ATintrongt100++;							
			} elsif ($line_item[3] eq "G" || $line_item[3] eq "C") {				
			    $interval_GCintrongt100++;
			} #end if			
		} elsif ($line_item[20] <= 100) {	
			if ($line_item[3] eq "A" || $line_item[3] eq "T") {		
				$interval_ATintronlt100++;				
			} elsif ($line_item[3] eq "G" || $line_item[3] eq "C") {
				$interval_GCintronlt100++;
			} #end if				
		} #end elsif
	} #end elsif
	
	} elsif ($first_interval==0) {
	    last;
	} #end if

} #end while

push(@GC_Intergene_count, $interval_GCintergene);
push(@AT_Intergene_count, $interval_ATintergene);

push(@GC_Intron_gt100count, $interval_GCintrongt100);
push(@AT_Intron_gt100count, $interval_ATintrongt100);

push(@GC_Intron_lt100count, $interval_GCintronlt100);
push(@AT_Intron_lt100count, $interval_ATintronlt100);

push(@GC_total_count, $GC_count);
push(@AT_total_count, $AT_count);

push(@Dper_mismatch, $per_mismatch);
push(@Dper_match, $per_match);

push(@Dmir_mismatch, $mir_mismatch);
push(@Dmir_match, $mir_match);

$interval_GCintergene = 0;
$interval_ATintergene = 0;
$interval_GCintrongt100 = 0;
$interval_ATintrongt100 = 0;
$interval_GCintronlt100 = 0;
$interval_ATintronlt100 = 0;
$GC_count = 0;
$AT_count = 0;
$per_mismatch = 0;
$per_match = 0;
$mir_mismatch = 0;
$mir_match = 0;

} #end for

#Header of output

print OUTPUT "interval start\tinterval end\t\#GC sites\t\#AT sites\t\#dper match\t\#Dper mismatch\t\#Dmir match\t\#Dmir mismatch\t\#IntergeneGC\t\#IntergeneAT\t\#intronGC gt 100\t\#intronAT gt 100\t\#intronGC lt 100\t\#intronAT lt 100\n";

#Print loop for the output

for ($n=0; $n<=$#start_pos; $n++) {
    print OUTPUT "$start_pos[$n]\t$stop_pos[$n]\t$GC_total_count[$n]\t$AT_total_count[$n]\t$Dper_match[$n]\t$Dper_mismatch[$n]\t$Dmir_match[$n]\t$Dmir_mismatch[$n]\t$GC_Intergene_count[$n]\t$AT_Intergene_count[$n]\t$GC_Intron_gt100count[$n]\t$AT_Intron_gt100count[$n]\t$GC_Intron_lt100count[$n]\t$AT_Intron_lt100count[$n]\n";
} #end for
