#! /usr/bin/perl

#a program to give GC3 count on all genes in an annotated chromosome file
#Defines command line arguments

$annotdata = $ARGV[0]; #command argument 1 is the annotation file
$intervaldata = $ARGV[1]; #command argument 2 is the pre-defined intervals
$output = $ARGV[2]; #command argument 3 is the output file name

open (ANNOTATION, $annotdata);
open (INTERVALS, $intervaldata);
open (OUTPUT, ">$output");

#Intialize necessary arrays.

@gene_start_array=();
@gene_end_array=();
@gene_name_array=();
@GC3_count_array=();
@AT3_count_array=();
$in_a_gene=0;
$first_line_of_gene=1;
$GC3_count=0;
$AT3_count=0;

#Cut the header off the annoation file.

$first_line=<ANNOTATION>;

#Read in the annotation file.

while (<ANNOTATION>) {

	$line = $_;
	@line_item=split(",",$line);

#Determine Intergene vs. Coding vs. Intron

	if ($line_item[19] ne "Intergene" && $first_line_of_gene==1) {

#Within a gene, first line, captures key information about the gene.

		$in_a_gene=1; #Defines in_a_gene as TRUE
  		$first_line_of_gene=0; #Defines first_line_of_gene as FALSE
		$gene_start=$line_item[1]; #Captures gene start position
		push(@gene_start_array, $gene_start);
		$gene_name=$line_item[17]; #Captures gene name
		push(@gene_name_array, $gene_name);
	} #end if

	elsif ($line_item[19] ne "Intergene" && $first_line_of_gene==0) {

#Within a gene, captures G/C and A/T counts of third position codons

			if ($line_item[19] eq "C3") {
				$base=$line_item[3];
				$gene_end=$line_item[1]; #Defines gene end (only used if next line specifies "Intergene").
				if ($base eq "A" || $base eq "T") {
					$AT3_count++;
				} elsif ($base eq "G" || $base eq "C") {
					$GC3_count++;
				} else {
				    next;
				} #end else
			} else {
				next;
			} #end else
    } #end elsif

    elsif ($line_item[19] eq "Intergene" && $in_a_gene==1) {

#Reached the end of a gene and captures key information

			if ($line_item[17]==$gene_name) {

				push(@gene_end_array, $gene_end);  #Captures gene end position
				push(@GC3_count_array, $GC3_count); #Captures the GC3 count
				push(@AT3_count_array, $AT3_count); #Captures the AT3 count
				$GC3_count=0; #Resets GC3 count for next gene
				$AT3_count=0; #Resets AT3 count for next gene
			} else {	
				print STDERR "gene names don\'t match\n"; #Makes sure gene names match!
			} #end else

		$in_a_gene=0; #Defines in_a_gene to FALSE
		$first_line_of_gene=1; #Defines first_line_of_gene as TRUE
    } #end elsif 
	} #end while

@start_pos=();
@stop_pos=();

#Read in intervals and the start and stop positions

while (<INTERVALS>) {

chomp($interval=$_);
@intervals = split("\t", $interval);

push(@start_pos, $intervals[0]);
push(@stop_pos, $intervals[1]);

} #end while

#Initialize necessary arrays

@GC3_per_interval = ();
@AT3_per_interval = ();
@geneaverage_perinterval = ();
@gene_count = ();

#For this j-loop repeats for each interval a total GC3 and AT3 count and and average GC3 for all gene.
#Also counts how many genes in each interval.

for ($j=0; $j<=$#start_pos; $j++) {

    @average_GC3_pergene = ();	
    $interval_GC3_count = 0;
    $interval_AT3_count = 0;
    $gene_averageGC3 = 0;
    $sum_averages = 0;
    $total_genes = 0;
    $average_GC3 = 0;

#For this k-loop repeats for every gene from the annotation file that is within the interval defined by the j-loop the counts and averages.

	for ($k=0; $k<=$#gene_name_array; $k++) {

		if ($start_pos[$j] <=  $gene_start_array[$k] && $stop_pos[$j] >= $gene_end_array[$k]) {
			$interval_GC3_count += $GC3_count_array[$k];
			$interval_AT3_count += $AT3_count_array[$k];
		
			if ($GC3_count_array[$k]==0 && $AT3_count_array[$k]==0) {
			    next;
			} else {		     
			    $gene_averageGC3 = $GC3_count_array[$k] / ($GC3_count_array[$k] + $AT3_count_array[$k]);
			} #end else	
			
			push(@average_GC3_pergene, $gene_averageGC3);

		} #end if	
	} #end for

#Once all genes in an interval have been accounted for, these arrays capture for each interval the counts

	push(@GC3_per_interval, $interval_GC3_count);
	push(@AT3_per_interval, $interval_AT3_count);

#This m-loop, it sums the averages of GC3

	for ($m=0; $m<=$#average_GC3_pergene; $m++) {
		$sum_averages += $average_GC3_pergene[$m];
	} #end for

#Computes the total number of genes

	$total_genes = $#average_GC3_pergene + 1;

#Divides average by total number of genes only if there are 1 or more genes in the interval

    if ($total_genes > 0) {
	$average_GC3 = $sum_averages / $total_genes;
    } else {
	$average_GC3 = 0
    }
	push(@geneaverage_perinterval, $average_GC3);
	push(@gene_count, $total_genes);

} #end for

#Print header

print OUTPUT "interval start\tinterval end\tnumber of genes\t\#GC3\t\#AT3\taverage GC3 per gene\n";

#Print loop 

for ($n=0; $n<=$#start_pos; $n++) {
    print OUTPUT "$start_pos[$n]\t$stop_pos[$n]\t$gene_count[$n]\t$GC3_per_interval[$n]\t$AT3_per_interval[$n]\t$geneaverage_perinterval[$n]\n";
} #end for
